@@ -8,14 +8,14 @@ use Search::Tools::TokenList;
 use Search::Tools::UTF8;
 use Carp;
 
-our $VERSION = '0.66';
+our $VERSION = '0.67';
 
 __PACKAGE__->mk_accessors(qw( re ));
 
 sub init {
     my $self = shift;
     $self->SUPER::init(@_);
-    $self->{re} ||= qr/\w+(?:[\'\-]\w+)*/;
+    $self->{re} ||= qr/\w+(?:[\'\-\.]\w+)*/;
     if ( $self->debug ) {
         $self->set_debug( $self->debug - 1 );    # XS debug a level higher
     }
@@ -142,7 +142,12 @@ Returns a TokenList object representin the Tokens in I<string>.
 I<string> is "split" according to the regex in re().
 
 I<heat_seeker> can be either a CODE reference or a regex object (qr//)
-to use for testing is_hot per token.
+to use for testing is_hot per token. An example CODE reference:
+
+ my $tokens = $tokenizer->tokenize('foo bar', sub { 
+    my ($token) = @_;
+    # do something with token during initial iteration
+ },);
 
 I<match_num> is the parentheses number to consider the matching token
 in the re() value. The default is 0 (the entire matching pattern).
